/*=============================================================================
    iSYSTEM Putchar(), Getchar() implementations using a shared memory buffer
    for winIDEA Terminal I/O
    
    CPU:   generic                                          
    Tool:  generic
    File:  iSystemIO.c
    
    Version:  1.0
    Date:     14/07/2015

  -----------------------------------------------------------------------------
  Copyright (c)2015 iSYSTEM AG, All rights reserved.
  -----------------------------------------------------------------------------
  Software that is described herein is for illustrative purposes only which 
  provides customers with programming information regarding the iSYSTEM products.  
  This software is supplied "AS IS" without any warranties of any kind, and 
  iSYSTEM and its licensor disclaim any and all warranties, express or implied, 
  including all implied warranties of merchantability, fitness for a particular 
  purpose and non-infringement of intellectual property rights.  
  iSYSTEM assumes no responsibility or liability for the use of the software, 
  conveys no license or rights under any patent, copyright, mask work right, 
  or any other intellectual property rights in or to any products. 
  iSYSTEM reserves the right to make changes in the software without notification. 
  iSYSTEM also makes no representation or warranty that such application will be 
  suitable for the specified use without further testing or modification.
  
  Permission to use, copy, modify, and distribute this software and its
  documentation is hereby granted, under iSYSTEMs' and its licensor's relevant 
  copyrights in the software, without fee, provided that it is used in conjunction 
  with iSYSTEM products. This copyright, permission, and disclaimer notice 
  must appear in all copies of this code.
  ==========================================================================*/
#include "iSystemIO.h"
/* buffers must be 2^n in size ( max 0x100) */
#define TWRBUF_SIZE 0x100 
#define TRDBUF_SIZE 0x02

/*
Define target and client pointers at the end of the buffers:
Target pointer: Penultimate element inc/dec by the char IO functions
Client pointer: Last element inc/dec by winIDEA Terminal IO
*/
#define TWRBUF_TPTR (g_TWBuffer[TWRBUF_SIZE + 0])
#define TWRBUF_CPTR (g_TWBuffer[TWRBUF_SIZE + 1])
#define TRDBUF_TPTR (g_TRBuffer[TRDBUF_SIZE + 0])
#define TRDBUF_CPTR (g_TRBuffer[TRDBUF_SIZE + 1])

/* buffer manipulation helpers */
#define TWRBUF_FULL()  (TWRBUF_TPTR == ((TWRBUF_CPTR - 1) & (TWRBUF_SIZE - 1)))
#define TRDBUF_EMPTY() (TRDBUF_TPTR == TRDBUF_CPTR)

/* buffers */
static volatile unsigned char g_TWBuffer[TWRBUF_SIZE + 2];
static volatile unsigned char g_TRBuffer[TRDBUF_SIZE + 2];

/* g_TConn!=0 when debugguer is connected */
static volatile char g_TConn=0xff; 


/* Character Output */

void iSystemPutch(char c)
{
  while(TWRBUF_FULL())
  {
	//If Terminal is not connected return
    if(!g_TConn)
    {
      return;
    }
  }
 
  // Output character and increment the write target pointer
  g_TWBuffer[TWRBUF_TPTR] = c;
  TWRBUF_TPTR = (++TWRBUF_TPTR & (TWRBUF_SIZE-1));
}

/* Character Input */

int AsystKbhit()
{
  if(!g_TConn)
  {
    return 0;
  }
  return !TRDBUF_EMPTY(); 
}
 
char iSystemGetch(void)
{
  char nChar;
  
  // Wait for keyboard input
  while(!AsystKbhit());
  
  // Input character from read buffer and increment read target pointer 		
  nChar = g_TRBuffer[TRDBUF_TPTR];
  TRDBUF_TPTR = (++TRDBUF_TPTR & (TRDBUF_SIZE-1));;
  return nChar;
}  

///////////////////////////////////////////////

