INPUT (& &)
OUTPUT_FORMAT("elf32-littlearm", "elf32-bigarm", "elf32-littlearm")

/* Entry Point */
ENTRY(Reset_Handler)

/* VA108xx memory map and location definitions*/
MEMORY {  
    VECTORS(RX) : ORIGIN = 0x00000000, LENGTH = 0x200    /* 512 */ 
    ROM(RX)     : ORIGIN = 0x00000200, LENGTH = 0x1FE00 /* 128k */
    RAM(RWX)    : ORIGIN = 0x10000000, LENGTH = 0x8000  /* 32k */ 
}
__RAM_START = 0x10000000;    /* Start of SRAM */
__RAM_END   = 0x10008000;    /* End of SRAM */

/* Stack location and size definitions */
__STACK_BASE = __RAM_END;    
__SP_INIT = __STACK_BASE;				
__STACK_SIZE = 0x400; 	

/* Heap location and size definitions */ 
__HEAP_SIZE = 0x400;		

/* Output section definitions */
SECTIONS
{
  /* Locate interrupt vector at the base of ROM */
  .interrupts :
  {
	  __VECTOR_TABLE = .;
    . = ALIGN(4);
    *(.text.vectors);  
    . = ALIGN(4);
  } >VECTORS

  /* All code sections will be located here */   
  .text :
  {
    . = ALIGN(4);
    __TEXT_START = .;
    *(.text)           /* .text sections (code) */
    *(.text*)          /* .text* sections (code) */
    *(.rodata)         /* .rodata sections (constants, strings, etc.) */
    *(.rodata*)        /* .rodata* sections (constants, strings, etc.) */
    *(.glue_7)         /* glue arm to thumb code */
    *(.glue_7t)        /* glue thumb to arm code */
    *(.eh_frame)

    KEEP (*(.init))
    KEEP (*(.fini))

    . = ALIGN(4);
    __TEXT_END = .;     
  } >ROM
  
  .ARM.extab : { 
    *(.ARM.extab* .gnu.linkonce.armextab.*) 
  } >ROM
  
  .ARM : {
      . = ALIGN(4);
      
      __EXIDX_START = .;
      *(.ARM.exidx*)
      __EXIDX_END = .;
      
      . = ALIGN(4);
      __ROM_AT = .;
  } >ROM

  /* All data sections will be located here */
  .data : AT(__ROM_AT)
  {
    __DATA_START = .; 
    . = ALIGN(4);
    _sdata = .;        /* create a global symbol at data start */
    *(.data)           /* .data sections */
    *(.data*)          /* .data* sections */

    . = ALIGN(4);
    __DATA_END = .;    
  } >RAM
  
  __DATA_SIZE = __DATA_END - __DATA_START;
  
  /* Uninitialized data section */
  . = ALIGN(4);
  .bss :
  {
    /* This is used by the startup in order to initialize the .bss section */
    __START_BSS = .;
	  PROVIDE ( __bss_start__ = __START_BSS );
    *(.bss)
    *(.bss*)
    *(COMMON)

    . = ALIGN(4);
    __END_BSS = .;
	  PROVIDE ( __bss_end__ = __END_BSS );
  } >RAM
  
  /* Heap and stack sections are located here */
  ._user_heap_stack :
  {
    . = ALIGN(4);
    PROVIDE ( end = . );
    PROVIDE ( _end = . );
	  __HEAP_ADDR = .;
    . = . + __HEAP_SIZE;
    . = . + __STACK_SIZE;
    . = ALIGN(4);
  } >RAM
  
  .ARM.attributes 0 : { *(.ARM.attributes) }
}

