/**************************************************************************//**
 * @file     startup_ARMCM0.s
 * @brief    CMSIS Core Device Startup File for
 *           ARMCM0 Device Series
 * @version  V5.00
 * @date     02. March 2016
 ******************************************************************************/
/*
 * Copyright (c) 2009-2016 ARM Limited. All rights reserved.
 *
 * SPDX-License-Identifier: Apache-2.0
 *
 * Licensed under the Apache License, Version 2.0 (the License); you may
 * not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an AS IS BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 * Adopted for Voragotech VA108xx 
 */

	.syntax	unified
	.arch	armv6-m

	.section .stack
	.align	3
#ifdef __STACK_SIZE
	.equ	Stack_Size, __STACK_SIZE
#else
	.equ	Stack_Size, 0x00000200
#endif
	.globl	__StackTop
	.globl	__StackLimit
__StackLimit:
	.space	Stack_Size
	.size	__StackLimit, . - __StackLimit
__StackTop:
	.size	__StackTop, . - __StackTop

	.section .heap
	.align	3
#ifdef __HEAP_SIZE
	.equ	Heap_Size, __HEAP_SIZE
#else
	.equ	Heap_Size, 0x00000400
#endif
	.globl	__HeapBase
	.globl	__HeapLimit
__HeapBase:
	.if	Heap_Size
	.space	Heap_Size
	.endif
	.size	__HeapBase, . - __HeapBase
__HeapLimit:
	.size	__HeapLimit, . - __HeapLimit

	.section .vectors
	.align 2
	.globl	__Vectors
__Vectors:
	.long	__StackTop            /* Top of Stack */
	.long	Reset_Handler         /* Reset Handler */
	.long	NMI_Handler           /* NMI Handler */
	.long	HardFault_Handler     /* Hard Fault Handler */
	.long	0                     /* Reserved */
	.long	0                     /* Reserved */
	.long	0                     /* Reserved */
	.long	0                     /* Reserved */
	.long	0                     /* Reserved */
	.long	0                     /* Reserved */
	.long	0                     /* Reserved */
	.long	SVC_Handler           /* SVCall Handler */
	.long	0                     /* Reserved */
	.long	0                     /* Reserved */
	.long	PendSV_Handler        /* PendSV Handler */
	.long	SysTick_Handler       /* SysTick Handler */

	/* External interrupts */
	.long OC0_IRQHandler   	        /* 0: TIM 0 ISR */ 
    .long VOR_TIM1_IRQHandler 	    /* 1: TIM 1 ISR */ 
    .long OC2_IRQHandler            /* 2: General */ 
    .long VOR_UART0_IRQHandler      /* 3: UART A ISR */ 
    .long VOR_UART1_IRQHandler      /* 4: UART B ISR */ 
    .long VOR_GPIO_IRQHandler   	  /* 5: GPIO ISR */ 
    .long VOR_SPI0_IRQHandler       /* 6: SPI A ISR */ 
    .long VOR_SPI1_IRQHandler       /* 7: SPI B ISR */ 
    .long VOR_SPI2_IRQHandler       /* 8: SPI C ISR */ 
    .long VOR_I2C0_MS_IRQHandler    /* 9: I2C A Master ISR */ 
    .long VOR_I2C0_SL_IRQHandler    /* 10: I2C A Slave ISR */ 
    .long VOR_I2C1_MS_IRQHandler    /* 11: I2C B Master ISR */ 
    .long VOR_I2C1_SL_IRQHandler    /* 12: I2C B Slave ISR */ 
    .long OC13_IRQHandler           /* 13: General */ 
    .long OC14_IRQHandler           /* 14: General */ 
    .long OC15_IRQHandler           /* 15: General */ 
    .long OC16_IRQHandler           /* 16: General */ 
    .long OC17_IRQHandler           /* 17: General */ 
    .long OC18_IRQHandler           /* 18: General */ 
    .long OC19_IRQHandler           /* 19: General */ 
    .long OC20_IRQHandler           /* 20: General */ 
    .long OC21_IRQHandler           /* 21: General */ 
    .long OC22_IRQHandler           /* 22: General */ 
    .long OC23_IRQHandler           /* 23: General */ 
    .long OC24_IRQHandler           /* 24: General */ 
    .long OC25_IRQHandler           /* 25: General */ 
    .long OC26_IRQHandler           /* 26: General */ 
    .long OC27_IRQHandler           /* 27: General */ 
    .long OC28_IRQHandler           /* 28: General */ 
    .long OC29_IRQHandler           /* 29: General */ 
    .long OC30_IRQHandler           /* 30: General */ 
    .long OC31_IRQHandler           /* 31: General */ 

	.size	__Vectors, . - __Vectors

	.text
	.thumb
	.thumb_func
	.align	1
	.globl	Reset_Handler
	.type	Reset_Handler, %function
Reset_Handler:
/*  Firstly it copies data from read only memory to RAM. There are two schemes
 *  to copy. One can copy more than one sections. Another can only copy
 *  one section.  The former scheme needs more instructions and read-only
 *  data to implement than the latter.
 *  Macro __STARTUP_COPY_MULTIPLE is used to choose between two schemes.  */

#ifdef __STARTUP_COPY_MULTIPLE
/*  Multiple sections scheme.
 *
 *  Between symbol address __copy_table_start__ and __copy_table_end__,
 *  there are array of triplets, each of which specify:
 *    offset 0: LMA of start of a section to copy from
 *    offset 4: VMA of start of a section to copy to
 *    offset 8: size of the section to copy. Must be multiply of 4
 *
 *  All addresses must be aligned to 4 bytes boundary.
 */
	ldr	r4, =__copy_table_start__
	ldr	r5, =__copy_table_end__

.L_loop0:
	cmp	r4, r5
	bge	.L_loop0_done
	ldr	r1, [r4]
	ldr	r2, [r4, #4]
	ldr	r3, [r4, #8]

.L_loop0_0:
	subs	r3, #4
	blt	.L_loop0_0_done
	ldr	r0, [r1, r3]
	str	r0, [r2, r3]
	b	.L_loop0_0

.L_loop0_0_done:
	adds	r4, #12
	b	.L_loop0

.L_loop0_done:
#else
/*  Single section scheme.
 *
 *  The ranges of copy from/to are specified by following symbols
 *    __etext: LMA of start of the section to copy from. Usually end of text
 *    __data_start__: VMA of start of the section to copy to
 *    __data_end__: VMA of end of the section to copy to
 *
 *  All addresses must be aligned to 4 bytes boundary.
 */
	ldr	r1, =__etext
	ldr	r2, =__data_start__
	ldr	r3, =__data_end__

	subs	r3, r2
	ble	.L_loop1_done

.L_loop1:
	subs	r3, #4
	ldr	r0, [r1,r3]
	str	r0, [r2,r3]
	bgt	.L_loop1

.L_loop1_done:
#endif /*__STARTUP_COPY_MULTIPLE */

/*  This part of work usually is done in C library startup code. Otherwise,
 *  define this macro to enable it in this startup.
 *
 *  There are two schemes too. One can clear multiple BSS sections. Another
 *  can only clear one section. The former is more size expensive than the
 *  latter.
 *
 *  Define macro __STARTUP_CLEAR_BSS_MULTIPLE to choose the former.
 *  Otherwise efine macro __STARTUP_CLEAR_BSS to choose the later.
 */
#ifdef __STARTUP_CLEAR_BSS_MULTIPLE
/*  Multiple sections scheme.
 *
 *  Between symbol address __copy_table_start__ and __copy_table_end__,
 *  there are array of tuples specifying:
 *    offset 0: Start of a BSS section
 *    offset 4: Size of this BSS section. Must be multiply of 4
 */
	ldr	r3, =__zero_table_start__
	ldr	r4, =__zero_table_end__

.L_loop2:
	cmp	r3, r4
	bge	.L_loop2_done
	ldr	r1, [r3]
	ldr	r2, [r3, #4]
	movs	r0, 0

.L_loop2_0:
	subs	r2, #4
	blt	.L_loop2_0_done
	str	r0, [r1, r2]
	b	.L_loop2_0
.L_loop2_0_done:

	adds	r3, #8
	b	.L_loop2
.L_loop2_done:
#elif defined (__STARTUP_CLEAR_BSS)
/*  Single BSS section scheme.
 *
 *  The BSS section is specified by following symbols
 *    __bss_start__: start of the BSS section.
 *    __bss_end__: end of the BSS section.
 *
 *  Both addresses must be aligned to 4 bytes boundary.
 */
	ldr	r1, =__bss_start__
	ldr	r2, =__bss_end__

	movs	r0, 0

	subs	r2, r1
	ble	.L_loop3_done

.L_loop3:
	subs	r2, #4
	str	r0, [r1, r2]
	bgt	.L_loop3
.L_loop3_done:
#endif /* __STARTUP_CLEAR_BSS_MULTIPLE || __STARTUP_CLEAR_BSS */

#ifndef __NO_SYSTEM_INIT
	bl	SystemInit
#endif

#ifndef __START
#define __START main
#endif
	bl	__START

	.pool
	.size	Reset_Handler, . - Reset_Handler

	.align	1
	.thumb_func
	.weak	Default_Handler
	.type	Default_Handler, %function
Default_Handler:
  /* Load the address of the interrupt control register into r3. */
  ldr r3, NVIC_INT_CTRL_CONST
  /* Load the value of the interrupt control register into r2 from the
  address held in r3. */
  ldr r2, [r3, #0]
  /* The interrupt number is in the least significant byte - clear all
  other bits. */
  uxtb r2, r2
Infinite_Loop:
  /* Now sit in an infinite loop - the number of the executing interrupt
  is held in r2. */
  b  Infinite_Loop
	.size	Default_Handler, . - Default_Handler
  
.align 4
/* The address of the NVIC interrupt control register. */
NVIC_INT_CTRL_CONST: .word 0xe000ed04



/*    Macro to define default handlers. Default handler
 *    will be weak symbol and just dead loops. They can be
 *    overwritten by other handlers */
	.macro	def_irq_handler	handler_name
	.weak	\handler_name
	.set	\handler_name, Default_Handler
	.endm

	def_irq_handler	NMI_Handler
	def_irq_handler	HardFault_Handler
	def_irq_handler	SVC_Handler
	def_irq_handler	PendSV_Handler
	def_irq_handler	SysTick_Handler
	
    def_irq_handler OC0_IRQHandler   	      /* 0: TIM 0 ISR */ 
    def_irq_handler VOR_TIM1_IRQHandler 	  /* 1: TIM 1 ISR */ 
    def_irq_handler OC2_IRQHandler            /* 2: General */ 
    def_irq_handler VOR_UART0_IRQHandler      /* 3: UART A ISR */ 
    def_irq_handler VOR_UART1_IRQHandler      /* 4: UART B ISR */ 
    def_irq_handler VOR_GPIO_IRQHandler   	  /* 5: GPIO ISR */ 
    def_irq_handler VOR_SPI0_IRQHandler       /* 6: SPI A ISR */ 
    def_irq_handler VOR_SPI1_IRQHandler       /* 7: SPI B ISR */ 
    def_irq_handler VOR_SPI2_IRQHandler       /* 8: SPI C ISR */ 
    def_irq_handler VOR_I2C0_MS_IRQHandler    /* 9: I2C A Master ISR */ 
    def_irq_handler VOR_I2C0_SL_IRQHandler    /* 10: I2C A Slave ISR */ 
    def_irq_handler VOR_I2C1_MS_IRQHandler    /* 11: I2C B Master ISR */ 
    def_irq_handler VOR_I2C1_SL_IRQHandler    /* 12: I2C B Slave ISR */ 
    def_irq_handler OC13_IRQHandler           /* 13: General */ 
    def_irq_handler OC14_IRQHandler           /* 14: General */ 
    def_irq_handler OC15_IRQHandler           /* 15: General */ 
    def_irq_handler OC16_IRQHandler           /* 16: General */ 
    def_irq_handler OC17_IRQHandler           /* 17: General */ 
    def_irq_handler OC18_IRQHandler           /* 18: General */ 
    def_irq_handler OC19_IRQHandler           /* 19: General */ 
    def_irq_handler OC20_IRQHandler           /* 20: General */ 
    def_irq_handler OC21_IRQHandler           /* 21: General */ 
    def_irq_handler OC22_IRQHandler           /* 22: General */ 
    def_irq_handler OC23_IRQHandler           /* 23: General */ 
    def_irq_handler OC24_IRQHandler           /* 24: General */ 
    def_irq_handler OC25_IRQHandler           /* 25: General */ 
    def_irq_handler OC26_IRQHandler           /* 26: General */ 
    def_irq_handler OC27_IRQHandler           /* 27: General */ 
    def_irq_handler OC28_IRQHandler           /* 28: General */ 
    def_irq_handler OC29_IRQHandler           /* 29: General */ 
    def_irq_handler OC30_IRQHandler           /* 30: General */ 
    def_irq_handler OC31_IRQHandler           /* 31: General */ 


	.end
